@@ -12,7 +12,7 @@ $sipc->connect(
    Proto    => 'tcp',
 ) or die $sipc->errstr($@);
 
-warn "client connected to server\n";
+$sipc->debug(1);
 
 $sipc->send_raw("Hello server, gimme some data :-)\n") or die $sipc->errstr($!);
 my $answer = $sipc->read or die $sipc->errstr($!);
